/////////////////////////////////////////////////////////////////////////////
// @doc ButtonCtrl
// @module ButtonCtrl.Java | Implementation of the <c ButtonCtrl> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: ButtonCtrl.java,v 1.10 2001/05/16 18:37:42 assefa Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: ButtonCtrl.java,v $
// Revision 1.10  2001/05/16 18:37:42  assefa
// Clean up tooltip threads.
//
// Revision 1.9  2000/11/09 14:50:59  assefa
// Correct bug when displaying tooltips for horizontal toolbar placed top.
//
// Revision 1.8  2000/09/19 21:59:20  assefa
// Add support for CURSOR_TYPE parameter.
//
// Revision 1.7  2000/08/18 20:03:12  assefa
// Initialize member variables.
//
// Revision 1.6  2000/08/18 14:20:39  assefa
// Add tool tips support.
//
// Revision 1.5  2000/05/12 01:51:29  daniel
// Added TB_BUT_?_TARGET and TB_BUT_?_JSACTION
//
// Revision 1.4  2000/04/18 22:19:10  daniel
// Removed extra '}' brackets at end of file
//
// Revision 1.3  2000/04/06 11:26:19  gparent
// Fixed a drawing bug using default selection
//
// Revision 1.2  2000/03/30 03:16:41  gparent
// Added the default button.
//
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////

// ==========================================
//  Import modules
// ==========================================
// ******************************************************
import java.applet.Applet;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.AWTEventMulticaster;
import java.lang.Integer;
import java.lang.NumberFormatException; 
import java.lang.String;

import java.awt.Dimension;
import java.awt.Point;

import java.awt.Cursor;

/** ******************************************************
 *  This super class is a generic button.  It implements the general 
 * handling of a press button.  Using parameters in the applet, it could
 * load an image likes a .gif or .jpg, show a hint ( for now, it will show
 * hint message into the status bar when the cursor pass over the button.
 * Later, it will show the hint into tooltip window ) and some other parameters.  
 * 
 * All parameters are defined in  <a href="http://www.dmsolutions.on.ca/rosa/rosa_params.html">
 * Rosa applet parameters</a>
 * 
 * @author  Guy Parent
 *******************************************************/
public abstract class ButtonCtrl extends Component
{ 

// --------------------------------------------------------------------------
// Constants.
// --------------------------------------------------------------------------
    /** Constant - First part of all button parameters.  */
    final static protected  String kstrPrefixParam  = "TB_BUT_";
    /** Constant - Used to extract the hint string parameters    */
    final protected         String kstrHintParam        = "_HINT";
    /** Constant - Used to extract the cursor type string parameters    */
    final protected         String kstrCursorTypeParam        = "_CURSOR_TYPE";
    /** Constant - Used to extract the border value parameters   */
    final protected         String kstrBorderParam  = "_BORDER";
    /** Constant - Used to extract the XY coordinate parameters  */
    final protected         String kstrXYLocParam   = "_XY";
    /** Constant - Used to extract the input string parameters   */
    final protected         String kstrInputParam   = "_INPUT";
    /** Constant - Used to extract the action name string parameters     */
    final protected         String kstrNameParam    = "_NAME";
    /** Constant - Used to extract the action value string parameters    */
    final protected         String kstrValueParam   = "_VALUE";
    /** Constant - Used to extract the optional button target parameter  */
    final protected         String kstrTargetParam  = "_TARGET";
    /** Constant - Used to extract the optional JSAction parameter  */
    final protected         String kstrJSActionParam= "_JSACTION";
// --------------------------------------------------------------------------
// Attributs
// --------------------------------------------------------------------------
    /** Button name. This string will be use to get all other parameter from the html page.*/
    protected String                    m_strName = null;              
    /** Button Size including the offset. */
    protected int                       m_nXSize;                   
    protected int                       m_nYSize; 
    /** True if the button is pressed. */
    protected boolean               m_bPressed;
    /** Default image. This image will be draw when the button still unpress.  A "X" will be draw
     *   when the image in parameter will not able to load. */
    protected Image             m_Img;
    /** Object listener the action. This object manage the button action */
    protected ActionListener        m_Listener;             
    /** Command that should be send on the pressed action */
    protected String                    m_strCmd;               
    /** Help context. This string will be draw when the user pass over the button */
    protected String                    m_strHint = null;    
    /** String conating the cursor type 
     * defined cursors are : 
     * - "CROSSHAIR"
     * - "WAIT"
     * - "HAND"
     */
    protected String                    m_strCursorType = null;     
    /** Border color... Not used yet */
    protected int                       m_borderColor = 0;
    /** Name of the form parameter to set when the button is pressed. */
    protected String                    m_strNameParam;
    /** Specifies the value to set in the form parameter specified by the name */
    protected String                    m_strValueParam;
    /** Submit Target. Optional - Target value to set on the form if this button triggers a submit. */
    protected String                    m_strTarget;
    /** JavaScript Action. Optional - JavaScript command to execute if this button triggers a submit. */
    protected String                    m_strJSAction;
    /** Use to set the status bar */
    protected Rosa2000              m_applet;
    /** Use to know if the position for this button is take by the button manager. */
    protected boolean               m_bIsAutoPlace = true;
    /** tool tip */
    private    ToolTipButton            m_ToolTip = null;
    

    
    
    /** 
     *  Constructors,  Create an initialize the button.  The button could be place in
     *  a specific location or in a toolbar. It set also the listener.
     *@param    strName Name of the button. This string will be use to extract all other
     *              associated parameters.
     *@param    bAutoPress If true, the button will be press up in the release button action.
     *              Otherwise, it will live press down until the applet call setPressed method.
     *@param    applet Applet where the current button is created.  It will be used 
     *              to set the status bar information likes tooltip.
     */
    public ButtonCtrl(String strName,  boolean bAutoPress, Rosa2000 applet ) 
    { 
        this( strName, 0, 0, bAutoPress, applet );
    } 
    
    /**
     *  Constructors Create an initialize the button.  The button could be place in
     *  a specific location or in a toolbar. It set also the listener. 
     *@param    strName Name of the button. This string will be use to extract all other
     *              associated parameters.
     *@param nXSize Width of the button.
     *@param nYSize Height of the button.
     *@param bAutoPress If true, the button will be press up in the release button action.
     *            Otherwise, it will live press down until the applet call setPressed method.
     *@param applet Applet where the current button is created.  It will be used 
     *            to set the status bar information likes tooltip.
     */
    public ButtonCtrl(String strName,  int nXSize, int nYSize, boolean bAutoPress, Rosa2000 applet ) 
    { 
        m_nXSize = nXSize; 
        m_nYSize = nYSize; 
        m_strName = strName;
        m_bPressed = false; 
        addMouseListener( new ButtonHandler(this, bAutoPress ) ); 
        m_ToolTip = new ToolTipButton( applet, this );
        addMouseListener(m_ToolTip); 
        m_strCmd = ""; 
        m_applet = applet;
    } 

// ===============================================
// Virtual methods.
// ===============================================
    
    /**
     * This method is called to repaint the button. It must be override.
     */
    public abstract void paint( Graphics g);
    
// ===============================================
// Get/Set methods.
// ===============================================

	/**
	 * Get the name of the current button.
	 */
	public String getButtonName()
	{
		return m_strName;
	}
	
    /**
     * Get the current value in the m_bIsAutoPlace attributs.
     * @return <b>boolean</b> Return true if the button should be place automaticaly
     *                        in the toolbar.  This value is false when the parameter
     *                        TB_BUT_?_XY is set.
     */
    public boolean isAutoPlacement()
    {
        return m_bIsAutoPlace;
    }
    
    /**
     *  Set the current value in the m_bIsAutoPlace attributs.
     * @param   bAuto If true, the button will be place in the toolbar.
     */
    public void setAutoPlacement( boolean bAuto )
    {
        m_bIsAutoPlace = bAuto;
    }
    
    /**
     *  Return the action name in the form parameter. A value will be set in the submit operation.
     */
    public String getButtonActionName()
    {
        return m_strNameParam;
    }
    
    /**
     *  Return the action value attached to the action name in the form.
     */
    public String getButtonActionValue()
    {
        return m_strValueParam;
    }
    
    /**
     *  Return the string to put in the tooltip. It will be draw when the user pass over the
     *  button.
     */
    public String getHint()
    {
        return m_strHint;
    }

    /**
     * Return string of the first part of all button parameters.
     */
    public static String getPrefixButtonParam()
    {
        return kstrPrefixParam;
    }

    /**
     * Set the button in the pressed state.
     */
    public void setPressed( boolean bPressed )
    {
            m_bPressed = bPressed;
            repaint();
    }

    /**
     * Return <b>true</b> if the button is in pressed state.
     */
    public boolean isPressed( )
    {
            return m_bPressed;
    }
    
    /**
     * Return the active ActionListener
     * @see ActionListener
     */
    public ActionListener getListener( )
    {
            return m_Listener;
    }
    
    /**
     * Repaint the button.
     */
    public void update(Graphics g) 
    { 
        paint(g); 
    } 

    /**
     * Gets the preferred size of this component
     * @return A dimension object indicating this component's preferred size. 
     */
    public Dimension getPreferredSize() 
    { 
        return new Dimension(m_nXSize, m_nYSize); 
    } 

    /**
     * Set the command string. This command will be used when the action is 
     * perform.
     */
    public void setActionCommand(String s)
    { 
        m_strCmd = s; 
    } 

    /**
     * Get the action command string.
     */
    public String getActionCommand() 
    { 
        return m_strCmd; 
    } 

    /**
     * Returns the optional target value to set in the form if this button
     * triggers a submit.
     */
    public String getTarget()
    {
        return m_strTarget;
    }

    /**
     * Returns the optional JavaScript JSACTION command to execute if this 
     * button triggers a submit.
     */
    public String getJSAction()
    {
        return m_strJSAction;
    }

    /**
     * Adds the specified action listener to receive action events from this button. 
     * Action events occur when a user presses or releases the mouse over this button.
     * @param l The action listener.
     */
    public void addActionListener(ActionListener l)
    { 
        m_Listener = AWTEventMulticaster.add(m_Listener,l); 
    } 

    /**
     * Removes the specified action listener so that it no longer receives action events
     * from this button. Action events occur when a user presses or releases the mouse 
     * over this button.
     * @param l The action listener.
     */
    public void removeActionListener(ActionListener l) 
    { 
        m_Listener = AWTEventMulticaster.remove(m_Listener, l); 
    } 

    
// ===============================================
// Protected methods. ( Initializations )
// ===============================================
    /**
     * This method start the initialization.  Read all associated parameters from the 
     * applet.
     */
    protected void init( )
    {
        try
        {   
            if( m_strName != null )
            {
                // Get default image button.
                String strDefBut = kstrPrefixParam + m_strName;

                // Set the size of the button
                if( m_nXSize == 0 )
                    m_nXSize = m_Img.getWidth( m_applet ); 
                if( m_nYSize == 0 )
                    m_nYSize = m_Img.getHeight( m_applet ); 

                setSize( m_nXSize, m_nYSize );

                // Get Hint.
                m_strHint = m_applet.getParameter( strDefBut + kstrHintParam );

                // Get Cursor_type.
                m_strCursorType = m_applet.getParameter( strDefBut + kstrCursorTypeParam );

                // GetBorder
                readBorder();
            
                // Get the location.
                readLocation();
            
                // Get Input 
                m_strCmd = m_applet.getParameter( strDefBut + kstrInputParam );
                if( m_strCmd == null )
                    System.err.println( "Error - The parameter " + strDefBut + kstrInputParam + " is missing." +
                                        "This button will not be attached to any form paramter." );
                
                // Get Name
                m_strNameParam = m_applet.getParameter( strDefBut + kstrNameParam );
                if( m_strNameParam == null )
                    System.err.println( "Warning - The parameter " + strDefBut + kstrNameParam + " is missing. " +
                                        "This button will not be attached to any form paramter." );
            
                // Get value
                m_strValueParam = m_applet.getParameter( strDefBut + kstrValueParam );
                if( m_strValueParam == null )
                    System.err.println( "The parameter " + strDefBut + kstrValueParam + " is missing." +
                                        " No operation available." );

                // Get submit target (optional).
                m_strTarget = m_applet.getParameter( strDefBut + kstrTargetParam );
                // Get JSAction command (optional).
                m_strJSAction = m_applet.getParameter( strDefBut + kstrJSActionParam );
            
            }
            else
            {   // It should be impossible.
                System.err.println("Error, cannot initialize the current button. The name is missing.");    
                System.exit(1);
            }
        }
        catch(Exception e)
        {
            System.err.println("Error loading image.");
        }
    }

    /**
     * Read the border parameter. For now the current parameter is not used yet.
     */
    protected void readBorder()
    {
        // Get default image button.
        String strDefBut = kstrPrefixParam + m_strName;
        
        String strColorName = m_applet.getParameter( strDefBut +  kstrBorderParam );
        if( strColorName != null )
        {
            try
            {
                m_borderColor = Integer.valueOf( strColorName, 10 ).intValue();
                // debug
                if( Rosa2000.isDebugMode() == true )
                    System.out.println(strDefBut +  kstrBorderParam +" = " + strColorName );
            }
            catch( NumberFormatException e )
            {
                System.err.println( "Invalid value in the parameter " + strDefBut +  kstrBorderParam + ". The value \n" +
                                            "will be set to " + m_borderColor );
            }
        }
    }   

    /**
     * Read the location of the current button. If it found XY coordinate, the button
     * will be draw at this place. Otherwise the attribut { @link #m_bIsAutoPlace m_bIsAutoPlace }
     * will be set to <b>true </b>. Hence, the current button will be automaticaly place after the
     * latest button insert in the toolbar.
     * @see #m_bIsAutoPlace
     * @see ToolbarPanel
     */
    protected void readLocation()
    {
            // debug
        if( Rosa2000.isDebugMode() == true )
            System.out.println("ButtonCtrl::ReadLocation" );

        // Get the XY parameter
        String strLocation = m_applet.getParameter( kstrPrefixParam + m_strName + kstrXYLocParam );
        if( strLocation != null )
       {
            boolean bError = true;
            String strCoor = new String();
            int nX = 0, 
                 nY = 0;
            
            int nIndex = 0;
            // find the comma between X,Y coordinate
            if( ( nIndex = strLocation.indexOf(",", nIndex++ ) ) != -1 )
            {
                strCoor = strLocation.substring( 0, nIndex );
                try
                {
                    // get the X value
                    nX = Integer.valueOf( strCoor, 10 ).intValue();
                
                    /// Get the Y value                 
                    if( nIndex < strLocation.length()-1 )
                    {
                        strCoor = strLocation.substring( nIndex+1 );

                        if( strCoor != null )
                        {
                            // get the number
                            nY = Integer.valueOf( strCoor, 10 ).intValue();
                            bError = false;
                        }
                    }
                }
                catch( NumberFormatException e )
                {
                    ;// do nothing the error already true
                }
            }
    
            // Set the coordinate if no error.
            if( bError == false )
            {
                m_bIsAutoPlace = false;
                setLocation ( nX, nY );
            }
            else
            {
                System.err.println( "Invalid coordinate parameter: " + strLocation + 
                                             " for the button " + kstrPrefixParam + m_strName + kstrXYLocParam );
                m_bIsAutoPlace = true;
            }
            
        // debug
            if( Rosa2000.isDebugMode() == true )
                System.out.println("ButtonCtrl::ReadLocation = " + nX + " " + nY );
        }
    
    }

    /**
     * Return the X, Y position of the tool tip hint position.
     * @see ToolTipButton
     */
    public Point getHintPosition()
    {
        Point oPos = new Point(0, 0);
        
        if (false)//getParent() == m_applet)
        {
            //_TO_DO_
        }
        else //parent is tool bar
        {
            if (((ToolbarPanel)getParent()).IsVertical())
            {
                if (((ToolbarPanel)getParent()).IsLeft())
                {
                    oPos.x = getSize().width + 3;
                    oPos.y = getParent().getLocation().y + getLocation().y +
                    (getSize().height /4);
                    
                }
                else
                {
                    oPos.x = getParent().getLocation().x -  
                        m_ToolTip.getSize().width -3;
                    oPos.y = getParent().getLocation().y + getLocation().y +
                    (getSize().height/4);
                }
                // make sure that we do not go out of the applet's bounds
                if (oPos.y + m_ToolTip.getSize().height > 
                    m_applet.getBounds().height)
                {
                    oPos.y =  m_applet.getBounds().height -  
                        m_ToolTip.getSize().height;
                }   
            }
            else //horizontal
            {
                if (((ToolbarPanel)getParent()).IsTop())
                {
                    oPos.x = getParent().getLocation().x + getLocation().x +
                        (getSize().width/4);

                    if (oPos.x + m_ToolTip.getSize().width > 
                        m_applet.getBounds().width)
                      oPos.x =  m_applet.getBounds().width -  
                          m_ToolTip.getSize().width;

                    oPos.y = getSize().height + 3;
                    System.out.println(oPos.x + " , " + oPos.y);
                } 
                else
                {
                    oPos.x = getParent().getLocation().x + getLocation().x +
                        (getSize().width/4);
                    if (oPos.x + m_ToolTip.getSize().width > 
                        m_applet.getBounds().width)
                      oPos.x =  m_applet.getBounds().width -  
                          m_ToolTip.getSize().width;

                    oPos.y = getParent().getLocation().y - 
                        m_ToolTip.getSize().height - 3;
                }
            }
        }
        return oPos;
    }

    /**
     * Check if the parameter CURSOR_TYPE is set and valid. If yes set the
     * cursor to the one defined.
     * @see setCursorToDefault
     */
    public void setCursorType()
    {
        if (m_strCursorType != null)
        {
            if (m_strCursorType.compareTo(new String("CROSSHAIR")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
            else if (m_strCursorType.compareTo(new String("WAIT")) ==0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            else if (m_strCursorType.compareTo(new String("HAND")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            else if (m_strCursorType.compareTo(new String("MOVE")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
            else if (m_strCursorType.compareTo(new String("TEXT")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
            else if (m_strCursorType.compareTo(new String("E_RESIZE")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
            else if (m_strCursorType.compareTo(new String("N_RESIZE")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
            else if (m_strCursorType.compareTo(new String("NE_RESIZE")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR));
            else if (m_strCursorType.compareTo(new String("NW_RESIZE")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR));
            else if (m_strCursorType.compareTo(new String("S_RESIZE")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
            else if (m_strCursorType.compareTo(new String("SE_RESIZE")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
            else if (m_strCursorType.compareTo(new String("SW_RESIZE")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR));
            else if (m_strCursorType.compareTo(new String("W_RESIZE")) == 0)
              m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
        } 
        else setCursorToDefault();
    }
    
    /**
     * Set the cursor to the defult one (pointer).
     * @see setCursorType
     */
    public void setCursorToDefault()
    {
        m_applet.getMapObj().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

    /**
     * Clean up all the buttons (delete tooltip threads).
     */
    public void cleanUp()
    {
        if (m_ToolTip != null)
        {
            //System.out.println("buttonctrl:cleanUp");
            m_ToolTip.cleanUp();
        }
    }
}



