 /////////////////////////////////////////////////////////////////////////////
// @doc DrawObj
// @module DrawObj.Java | Implementation of the <c DrawObj> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: DrawObj.java,v 1.2 2000/08/18 14:23:17 assefa Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: DrawObj.java,v $
// Revision 1.2  2000/08/18 14:23:17  assefa
// Use double buffering when drawing (rectangle, circle..) instead of XOR.
//
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////

import java.awt.Color;
import java.awt.Point;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Component;

/**
 * This super class is a generic drawing object.  All drawing object
 * contains in the map must be inherit from DrawObj.
 *
 * @author  Guy Parent
 */
public abstract class DrawObj extends Component
{
    /** Color used to draw the object.   */
    protected Color     color;
    /** Graphic object use to draw.  */
    protected Graphics g = null;

    /** 
     * Set the current color.
     * @param color
     * @see Color
     */
    public void setColor(Color color)
    {   
        this.color = color; 
    }

    /**
     * Return the current color.
     */
    public Color getColor()
    {   
        return (color);     
    }

    /**
     * Destroy all listener or object event. It must be overload.
     */
    public abstract void  destroyEvent();

    /**
     * add all listener or object event. It must be overload.
     */
    public abstract void addEvent();

    /**
     * Repaint all drawing object.  This method will be called after an invalidate region.
     */
    public void paint( Graphics mapG ) 
    {
        draw();
    }
    /**
     * This method will be called at the end of operation. It must return the value which it
     * put in the value parameter in the submit operation.
     * @return String containing the submit value.
     * @see <a href="http://www.dmsolutions.on.ca/rosa/rosa_params.html"> Rosa applet parameters</a>
     */
    public abstract String getSubmitValue();
    
    /**
     * Used to draw the drawing object. In general, it will call on mouse move.  It must be 
     * overload.
     */
    public abstract void draw (  );
    /**
     * This method is called when the drawing object change size.
     * @param p new coordinate.
     */
    public abstract void resize ( Point p);

    /**
     * Used to disable the object : remove the events and do not draw.
     */
    public void disable_object()
    {
         reset_object();
         destroyEvent();
    }
    
    /**
     * Used to enable the object : add the events and prepare to draw.
     */
    public void enable_object()
    {
        addEvent();
    }

    public void reset_object(){};
}
